﻿using AbstractSoftwareInstallationContracts.BindingModels;
using AbstractSoftwareInstallationContracts.SearchModels;
using AbstractSoftwareInstallationContracts.StoragesContracts;
using AbstractSoftwareInstallationContracts.ViewModels;
using AbstractSoftwareInstallationDatabaseImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AbstractSoftwareInstallationDatabaseImplement.Implements
{
    public class ClientStorage : IClientStorage
    {
        public ClientViewModel? GetElement(ClientSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Email) && !model.Id.HasValue)
            {
                return null;
            }
            using var context = new AbstractSoftwareInstallationDatabase();
            return context.Clients
                .FirstOrDefault(x => (!string.IsNullOrEmpty(model.Email)
                        && x.Email == model.Email)
                    || (model.Id.HasValue && x.Id == model.Id))
                ?.GetViewModel;
        }

        public List<ClientViewModel> GetFilteredList(ClientSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Email))
            {
                return new();
            }
            using var context = new AbstractSoftwareInstallationDatabase();
            return context.Clients
                .Where(x => x.Email.Contains(model.Email))
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public List<ClientViewModel> GetFullList()
        {
            using var context = new AbstractSoftwareInstallationDatabase();
            return context.Clients
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public ClientViewModel? Insert(ClientBindingModel model)
        {
            var newClient = Client.Create(model);
            if (newClient == null)
            {
                return null;
            }
            using var context = new AbstractSoftwareInstallationDatabase();
            context.Clients.Add(newClient);
            context.SaveChanges();
            return newClient.GetViewModel;
        }

        public ClientViewModel? Update(ClientBindingModel model)
        {
            using var context = new AbstractSoftwareInstallationDatabase();
            var Client = context.Clients.FirstOrDefault(x => x.Id == model.Id);
            if (Client == null)
            {
                return null;
            }
            Client.Update(model);
            context.SaveChanges();
            return Client.GetViewModel;
        }
        public ClientViewModel? Delete(ClientBindingModel model)
        {
            using var context = new AbstractSoftwareInstallationDatabase();
            var element = context.Clients.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Clients.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
