﻿using AbstractPackageInstallationDatabaseImplement.Models;
using AbstractSoftwareInstallationContracts.BindingModels;
using AbstractSoftwareInstallationContracts.SearchModels;
using AbstractSoftwareInstallationContracts.StoragesContracts;
using AbstractSoftwareInstallationContracts.ViewModels;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AbstractSoftwareInstallationDatabaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new AbstractSoftwareInstallationDatabase();
            var element = context.Orders.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Orders.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }

        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            using var context = new AbstractSoftwareInstallationDatabase();
            return context.Orders
                    .Include(x => x.Package)
                    .Include(x => x.Client)
                    .Include(x => x.Implementer)
                    .FirstOrDefault(x =>
                        (model.Status == null || model.Status != null && model.Status.Equals(x.Status)) &&
                        model.ImplementerId.HasValue && x.ImplementerId == model.ImplementerId ||
                        model.Id.HasValue && x.Id == model.Id)?.GetViewModel;
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (!model.Id.HasValue && !model.DateFrom.HasValue && !model.DateTo.HasValue && !model.ClientId.HasValue && model.Status == null)
            {
                return new();
            }
            using var context = new AbstractSoftwareInstallationDatabase();
            if (model.ClientId.HasValue)
            {
                return context.Orders
                    .Include(x => x.Client)
                    .Include(x => x.Implementer)
                    .Include(x => x.Package)
                    .Where(x => x.ClientId == model.ClientId)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            if (model.Status != null)
            {
                return context.Orders
                    .Include(x => x.Package)
                    .Include(x => x.Client)
                    .Include(x => x.Implementer)
                    .Where(x => model.Status.Equals(x.Status))
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            return context.Orders
                .Include(x => x.Package)
                .Include(x => x.Client)
                .Include(x => x.Implementer)
                .Where(x => x.Id == model.Id || model.DateFrom <= x.DateCreate && x.DateCreate <= model.DateTo)
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public List<OrderViewModel> GetFullList()
        {
            using var context = new AbstractSoftwareInstallationDatabase();
            return context.Orders
                .Include(x => x.Package).Include(x => x.Client).Include(x => x.Implementer)
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            var newOrder = Order.Create(model);

            if (newOrder == null)
            {
                return null;
            }
            using var context = new AbstractSoftwareInstallationDatabase();
            context.Orders.Add(newOrder);
            context.SaveChanges();
            return context.Orders.Include(x => x.Package).Include(x => x.Client).Include(x => x.Implementer).FirstOrDefault(x => x.Id == newOrder.Id)?.GetViewModel;
            
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new AbstractSoftwareInstallationDatabase();
            var order = context.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }
            order.Update(model);
            context.SaveChanges();
            return context.Orders.Include(x => x.Package).Include(x => x.Client).Include(x => x.Implementer).FirstOrDefault(x => x.Id == order.Id)?.GetViewModel;
        }
    }
}
