﻿using AbstractSoftwareInstallationContracts.BindingModels;
using AbstractSoftwareInstallationContracts.SearchModels;
using AbstractSoftwareInstallationContracts.StoragesContracts;
using AbstractSoftwareInstallationContracts.ViewModels;
using AbstractSoftwareInstallationFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AbstractSoftwareInstallationFileImplement.Implements
{
    public class ClientStorage : IClientStorage
    {
        private readonly DataFileSingleton _source;
        public ClientViewModel? GetElement(ClientSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Email) && !model.Id.HasValue)
            {
                return null;
            }
            return _source.Clients.FirstOrDefault(x =>
            (!string.IsNullOrEmpty(model.Email)
            && x.Email == model.Email)
            || (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }

        public List<ClientViewModel> GetFilteredList(ClientSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Email))
            {
                return new();
            }
            return _source.Clients.Where(x => x.Email.Contains(model.Email))
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public List<ClientViewModel> GetFullList()
        {
            return _source.Clients.Select(x => x.GetViewModel).ToList();
        }

        public ClientViewModel? Insert(ClientBindingModel model)
        {
            model.Id = _source.Clients.Count > 0 ? _source.Clients.Max(x => x.Id) + 1 : 1;
            var newClient = Client.Create(model);
            if (newClient == null)
            {
                return null;
            }
            _source.Clients.Add(newClient);
            _source.SaveClients();
            return newClient.GetViewModel;
        }

        public ClientViewModel? Update(ClientBindingModel model)
        {
            var Client = _source.Clients.FirstOrDefault(x => x.Id == model.Id);
            if (Client == null)
            {
                return null;
            }
            Client.Update(model);
            _source.SaveClients();
            return Client.GetViewModel;
        }
        public ClientViewModel? Delete(ClientBindingModel model)
        {
            var Client = _source.Clients.FirstOrDefault(x => x.Id == model.Id);
            if (Client == null)
            {
                return null;
            }
            _source.Clients.Remove(Client);
            _source.SaveClients();
            return Client.GetViewModel;
        }
    }
}
