﻿using AbstractClientInstallationListImplement.Models;
using AbstractSoftwareInstallationContracts.BindingModels;
using AbstractSoftwareInstallationContracts.SearchModels;
using AbstractSoftwareInstallationContracts.StoragesContracts;
using AbstractSoftwareInstallationContracts.ViewModels;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AbstractSoftwareInstallationListImplement.Implements
{
    public class ClientStorage : IClientStorage
    {
        private readonly DataListSingleton _source;
        public ClientStorage()
        {
            _source = DataListSingleton.GetInstance();
        }
        public ClientViewModel? Delete(ClientBindingModel model)
        {
            throw new NotImplementedException();
        }

        public ClientViewModel? GetElement(ClientSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ClientFIO) && !model.Id.HasValue)
            {
                return null;
            }
            foreach (var client in _source.Clients)
            {
                if ((!string.IsNullOrEmpty(model.ClientFIO) &&
               client.ClientFIO == model.ClientFIO) ||
                (model.Id.HasValue && client.Id == model.Id))
                {
                    return client.GetViewModel;
                }
            }
            return null;
        }

        public List<ClientViewModel> GetFilteredList(ClientSearchModel model)
        {
            var result = new List<ClientViewModel>();
            if (string.IsNullOrEmpty(model.ClientFIO))
            {
                return result;
            }
            foreach (var client in _source.Clients)
            {
                if (client.ClientFIO.Contains(model.ClientFIO))
                {
                    result.Add(client.GetViewModel);
                }
            }
            return result;
        }

        public List<ClientViewModel> GetFullList()
        {
            var result = new List<ClientViewModel>();
            foreach (var client in _source.Clients)
            {
                result.Add(client.GetViewModel);
            }
            return result;
        }

        public ClientViewModel? Insert(ClientBindingModel model)
        {
            model.Id = 1;
            foreach (var client in _source.Clients)
            {
                if (model.Id <= client.Id)
                {
                    model.Id = client.Id + 1;
                }
            }
            var newClient = Client.Create(model);
            if (newClient == null)
            {
                return null;
            }
            _source.Clients.Add(newClient);
            return newClient.GetViewModel;
        }

        public ClientViewModel? Update(ClientBindingModel model)
        {
            foreach (var client in _source.Clients)
            {
                if (client.Id == model.Id)
                {
                    client.Update(model);
                    return client.GetViewModel;
                }
            }
            return null;
        }
    }
}
